/*
 * tiffwrite.c
 * File output routines for minitiff.
 *
 * Copyright (C) 2006-2025 Cosmin Truta.
 *
 * Use, modification and distribution are subject
 * to the Boost Software License, Version 1.0.
 * See the accompanying file LICENSE_BSL_1_0.txt
 * or visit https://www.boost.org/LICENSE_1_0.txt
 *
 * SPDX-License-Identifier: BSL-1.0
 */

#include "minitiff.h"

#if 0
#error No file writing has been implemented yet.
#endif
